/*  Autopackage GTK+ frontend
 *  Copyright (C) 2003  Hongli Lai <h.lai@chello.nl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include "processor.h"
#include "protocol.h"
#include "utils.h"
#include "main.h"
#include "receiver.h"
#include "interactions.h"
#include "logview.h"
#include "eggstatusicon.h"

#define W(x) glade_xml_get_widget (xml, x)

#define PROTOCOL_VERSION "2"


static gboolean helloInitialized = FALSE;
static gboolean failed = FALSE;
static guint sessionDepth = 1;
static gboolean installStarted = FALSE;
static gboolean progressStarted = FALSE;
static guint maxPackages = 0;
static guint packagesInstalled = 0;
static GList *interactionList = NULL;

static void present_success (void);
static void present_failed (void);


static gboolean
process_hello (gchar *data)
{
	gchar *first, *tmp;

	/* Initial handshake for every package */

	tmp = strchr (data, '\n');
	first = g_strndup (data, tmp - data);

	if (my_strcmp (first + 6, PROTOCOL_VERSION))
	{
		/* Don't increase the session depth if this is the first HELLO. */
		if (helloInitialized)
			sessionDepth++;

		protocol_send ("LETS GO");
		g_free (first);
		helloInitialized = TRUE;

		return TRUE;
	} else
	{
		protocol_send ("UHOH VERSION_MISMATCH");
		show_error (W("main"),
			_("Protocol version mismatched (expected %s, but got %s).\n"
			"Please ensure your installation of Autopackage is correct."),
			PROTOCOL_VERSION, first + 6);
		g_free (first);
		return FALSE;
	}
}


static gboolean
process_terminate ()
{
	sessionDepth--;
	if (sessionDepth > 0)
		return TRUE;

	/* The global installation has terminated */

//	g_signal_connect (W("install"), "delete_event", G_CALLBACK (gtk_main_quit), NULL);
	pulse_stop ("globalProgress");

	/* We exited normally so no need to remove the FIFO */
	pokeFifo = FALSE;
	receiver_stop ();

	finished = TRUE;

	if (failed)
		present_failed ();
	else
		present_success ();
	return TRUE;
}


static gboolean
process_switch_group (gchar *data)
{
	/* Ignore this */
	return TRUE;
}


static gboolean
process_identify (gchar *data)
{
	GList *lines;
	gchar *displayName;
	guint id;
	gchar *context;
	gchar *msg;

	/* We are current processing a package */

	lines = create_strlist (data);
	displayName = (gchar *) g_list_nth_data (lines, 1);
	id = atoi ((gchar *) g_list_nth_data (lines, 2));
	context = (gchar *) g_list_nth_data (lines, 3);

	gtk_notebook_set_current_page (GTK_NOTEBOOK (W("notebook1")), 1);

	gtk_label_set_text (GTK_LABEL (W("name")), displayName);
	if (my_strcmp (context, "prep"))
	{
		GList *children;

		/* Preparing a package (checking dependancies) */
		gtk_label_set_markup (GTK_LABEL (W("title")),
			_("<span weight=\"bold\" size=\"x-large\">Preparing package:</span>"));
		gtk_widget_show (W("prepareImage"));
		gtk_widget_hide (W("installImage"));
		gtk_notebook_set_current_page (GTK_NOTEBOOK (W("notebook1")), 1);

		children = gtk_container_get_children (GTK_CONTAINER (
				GTK_LOG_VIEW (W("logview"))->vbox
			));
		if (g_list_first (children))
			gtk_log_view_add_separator (GTK_LOG_VIEW (W("logview")));
		g_list_free (children);

		set_title (_("Preparing package: %s"), displayName);

		gtk_log_view_clear (GTK_LOG_VIEW (W("logview")));

	} else if (my_strcmp (context, "install"))
	{
		/* Installing a package */

		gtk_label_set_markup (GTK_LABEL (W("title")),
			_("<span weight=\"bold\" size=\"x-large\">Currently installing:</span>"));
		gtk_widget_hide (W("prepareImage"));
		gtk_widget_show (W("installImage"));

		packagesInstalled = id;
		gtk_notebook_set_current_page (GTK_NOTEBOOK (W("notebook1")), 1);
		gtk_log_view_clear (GTK_LOG_VIEW (W("logview")));

		if (maxPackages > 1)
		{
			set_title (_("Installing: %s (package %d of %d)"),
				displayName, id, maxPackages,
				((gdouble) id / (gdouble) (maxPackages + 1)) * 100.0);

			msg = g_strdup_printf ("Package %d of %d", id, maxPackages);
			gtk_progress_bar_set_text (GTK_PROGRESS_BAR (W("globalProgress")), msg);
			g_free (msg);
			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (W("globalProgress")),
				(gdouble) id / (gdouble) (maxPackages + 1));
		} else
			set_title (_("Installing: %s"), displayName);
	}

	free_strlist (lines);
	return TRUE;
}


static gboolean
process_series (gchar *data)
{
	gchar *max;

	/* Get maximum number of packages */

	max = strchr (data, '\n') + 1;
	stripNewline (max);
	maxPackages = atoi (max);

	if (maxPackages > 1)
	{
		gchar *msg;

		gtk_widget_show (W("globalProgress"));
		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (W("globalProgress")), 0.0);
		msg = g_strdup_printf ("Preparing %d packages...", maxPackages);
		gtk_progress_bar_set_text (GTK_PROGRESS_BAR (W("globalProgress")), msg);
		g_free (msg);
	}

	installStarted = TRUE;
	return TRUE;
}


static gboolean
process_status (gchar *data)
{
	gchar *status;

	status = strchr (data, '\n') + 1;
	stripNewline (status);
	if (strlen (status) == 0)
		return TRUE;

	gtk_notebook_set_current_page (GTK_NOTEBOOK (W("notebook1")), 1);
	gtk_log_view_add_status (GTK_LOG_VIEW (W("logview")), status);
	return TRUE;
}


static gboolean
process_test (gchar *data)
{
	gchar *name, *str;

	name = strchr (data, '\n') + 1;
	stripNewline (name);
	if (strlen (name) == 0)
		return TRUE;

	gtk_notebook_set_current_page (GTK_NOTEBOOK (W("notebook1")), 1);
	str = g_strdup_printf (_("Checking for %s..."), name);
	gtk_log_view_check_begin (GTK_LOG_VIEW (W("logview")), str);
	g_free (str);
	return TRUE;
}


static gboolean
process_progressbar (gchar *data)
{
	GList *lines = create_strlist (data);
	gdouble max, current;
	gchar *label;

	max = (gdouble) atoi ((gchar *) g_list_nth_data (lines, 1));
	current = (gdouble) atoi ((gchar *) g_list_nth_data (lines, 2));
	label = (gchar *) g_list_nth_data (lines, 3);

	gtk_notebook_set_current_page (GTK_NOTEBOOK (W("notebook1")), 1);

	if (max == current)
	{
		/* The progress bar has reached it's end */
		gtk_log_view_set_progress (GTK_LOG_VIEW (W("logview")), 1.0);
		progressStarted = FALSE;
	} else
	{
		if (!progressStarted)
			gtk_log_view_add_progress (GTK_LOG_VIEW (W("logview")), label);
		gtk_log_view_set_progress (GTK_LOG_VIEW (W("logview")), current / max);
		progressStarted = TRUE;
	}

	free_strlist (lines);
	return TRUE;
}


static gboolean
process_fail (gchar *data)
{
	gchar *msg;

	msg = strchr (data, '\n') + 1;
	failed = TRUE;

	gtk_notebook_set_current_page (GTK_NOTEBOOK (W("notebook1")), 1);
	stripNewline (msg);
	gtk_log_view_add_fail (GTK_LOG_VIEW (W("logview")), msg);

	return TRUE;
}


static gboolean
process_passtest (gchar *data)
{
	gtk_notebook_set_current_page (GTK_NOTEBOOK (W("notebook1")), 1);
	gtk_log_view_check_pass (GTK_LOG_VIEW (W("logview")));
	return TRUE;
}


static gboolean
process_failtest (gchar *data)
{
	gtk_notebook_set_current_page (GTK_NOTEBOOK (W("notebook1")), 1);
	gtk_log_view_check_fail (GTK_LOG_VIEW (W("logview")));
	return TRUE;
}


static gboolean
process_recommendtest (gchar *data)
{
	gtk_notebook_set_current_page (GTK_NOTEBOOK (W("notebook1")), 1);
	gtk_log_view_check_recommended (GTK_LOG_VIEW (W("logview")));
	return TRUE;
}


static gboolean
process_interact (gchar *data)
{
	GList *lines;
	gchar *name, *message, *defaultValue;

	lines = create_strlist (data);
	name = (gchar *) g_list_nth_data (lines, 1);
	message = (gchar *) g_list_nth_data (lines, 2);
	defaultValue = (gchar *) g_list_nth_data (lines, 3);

	if (startsWith (data, "INTERACT STRING\n"))
		interactionList = interaction_add (interactionList,
			name, message, defaultValue, INTERACT_STRING);

	else if (startsWith (data, "INTERACT PATH\n"))
		interactionList = interaction_add (interactionList,
			name, message, defaultValue, INTERACT_PATH);

	free_strlist (lines);
	return TRUE;
}


static gboolean
process_wait (gchar *data)
{
	interaction_execute (interactionList);
	return TRUE;
}


static gboolean
process_get (gchar *data)
{
	gchar *name;

	name = strchr (data, '\n') + 1;
	stripNewline (name);
	protocol_send ((gchar *) interaction_get (interactionList, name));
	return TRUE;
}


gboolean
process (gchar *data)
{
	gboolean result = TRUE;

	if (startsWith (data, "HELLO "))
		return process_hello (data);

	if (startsWith (data, "TERMINATE\n"))
		result = process_terminate ();

	else if (startsWith (data, "SWITCH GROUP\n"))
		result = process_switch_group (data);

	else if (startsWith (data, "IDENTIFY\n"))
		result = process_identify (data);

	else if (startsWith (data, "SERIES\n"))
		result = process_series (data);

	else if (startsWith (data, "STATUS\n"))
		result = process_status (data);

	else if (startsWith (data, "TEST\n"))
		result = process_test (data);

	else if (startsWith (data, "PROGRESSBAR\n"))
		result = process_progressbar (data);

	else if (startsWith (data, "FAIL\n"))
		result = process_fail (data);

	else if (startsWith (data, "FAILTEST\n"))
		result = process_failtest (data);

	else if (startsWith (data, "PASSTEST\n"))
		result = process_passtest (data);

	else if (startsWith (data, "RECOMMEND\n"))
		result = process_recommendtest (data);

	else if (startsWith (data, "INTERACT "))
		result = process_interact (data);

	else if (startsWith (data, "WAIT\n"))
		result = process_wait (data);

	else if (startsWith (data, "GET\n"))
	{
		result = process_get (data);
		#ifdef DEBUG
		g_print ("\n");
		#endif
		return result;
	}

	else
	{
		show_error (W("install"),
			_("The backend sent an unknown command:\n%s"),
			data);
		exit (1);
	}

	protocol_send ("OK");
	/* #ifdef DEBUG
	g_print ("\n");
	#endif */
	return result;
}


/* Present an "installation successful" screen */
static void
present_success ()
{
	gchar *msg, *s;

	gtk_notebook_set_current_page (GTK_NOTEBOOK (W("notebook1")), 1);
	set_title (_("Installation completed"));

	pulse_stop ("globalProgress");
	gtk_widget_hide (W("globalProgress"));
	gtk_image_set_from_file (GTK_IMAGE (W("installImage")),
		lookup_file ("package-ok.png"));
	gtk_widget_hide (W("prepareImage"));
	gtk_widget_show (W("installImage"));

	gtk_label_set_text (GTK_LABEL (W("name")), "");
	gtk_label_set_markup (GTK_LABEL (W("title")),
		_("<span weight=\"bold\" size=\"x-large\">The installation has finished.</span>"));

	if (maxPackages <= 1)
		msg = g_strdup (_("Installation of the requested package is complete" ".\n"
			"Click on OK to exit."));
	else
		msg = g_strdup_printf (
			_("%d packages have been installed.\n"
			"Click on OK to exit."),
			maxPackages);
	gtk_log_view_clear (GTK_LOG_VIEW (W("logview")));
	//gtk_log_view_add_separator (GTK_LOG_VIEW (W("logview")));
	gtk_log_view_add_status (GTK_LOG_VIEW (W("logview")), msg);
	g_free (msg);
	
	gtk_widget_hide (W("hide"));
	gtk_widget_show (W("close"));
	gtk_button_set_label (GTK_BUTTON (W("close")), GTK_STOCK_OK);
	gtk_button_set_use_stock (GTK_BUTTON (W("close")), TRUE);
	gtk_widget_grab_default (W("close"));
	gtk_widget_grab_focus (W("close"));

	s = lookup_file ("package-open.png");
	if (!statusIcon) statusIcon = egg_status_icon_new();

	if (maxPackages == 1)
		msg = g_strdup_printf(_("Installation of the requested package is complete"), maxPackages);
	else
		msg = g_strdup_printf(_("Installation of %d packages is complete"), maxPackages);
	egg_status_icon_set_tooltip(statusIcon, msg, NULL);
	egg_status_icon_set_is_blinking (statusIcon, TRUE);
	egg_status_icon_set_from_file (statusIcon, s);
	g_signal_connect(G_OBJECT(statusIcon), "activate", G_CALLBACK(gtk_main_quit), NULL);
	g_free(s);
	g_free(msg);	
}


/* Present an "installation failed" screen */
static void
present_failed ()
{
	gtk_notebook_set_current_page (GTK_NOTEBOOK (W("notebook1")), 1);
	set_title (_("Installation failed"));

	pulse_stop ("globalProgress");

	gtk_image_set_from_stock (GTK_IMAGE (W("installImage")), GTK_STOCK_DIALOG_ERROR,
		GTK_ICON_SIZE_DIALOG);
	gtk_widget_hide (W("prepareImage"));
	gtk_widget_show (W("installImage"));

	gtk_label_set_text (GTK_LABEL (W("name")), "");
	gtk_label_set_markup (GTK_LABEL (W("title")),
		_("<span weight=\"bold\" size=\"x-large\">The installation has failed.</span>"));

	gtk_widget_hide (W("hide"));
	gtk_widget_show (W("close"));
	gtk_widget_grab_default (W("close"));
	gtk_widget_grab_focus (W("close"));

	if (silentMode == TRUE && statusIcon)
	{
		char *s = lookup_file ("package-open.png");

		/* FIXME: use a better icon */
		egg_status_icon_set_is_blinking (statusIcon, TRUE);
		egg_status_icon_set_from_file (statusIcon, s);
		g_free (s);
	}

	exitCode = 2;
}
